import {
  Lifetime,
  QuickJSMemoryLeakDetected
} from "./chunk-S2GAWW6L.mjs";

// src/index.ts
export * from "@jitl/quickjs-ffi-types";

// src/esmHelpers.ts
function actualUnwrapDefault(mod) {
  const maybeUnwrap = mod.default;
  return maybeUnwrap ?? mod;
}
var unwrapTypescript = actualUnwrapDefault;

// src/from-variant.ts
async function newQuickJSWASMModuleFromVariant(variantOrPromise) {
  const variant = smartUnwrap(await variantOrPromise);
  const [wasmModuleLoader, QuickJSFFI, { QuickJSWASMModule }] = await Promise.all([
    variant.importModuleLoader().then(smartUnwrap),
    variant.importFFI(),
    import("./module-LQK32M7Y.mjs").then(unwrapTypescript)
  ]);
  const wasmModule = await wasmModuleLoader();
  wasmModule.type = "sync";
  const ffi = new QuickJSFFI(wasmModule);
  return new QuickJSWASMModule(wasmModule, ffi);
}
async function newQuickJSAsyncWASMModuleFromVariant(variantOrPromise) {
  const variant = smartUnwrap(await variantOrPromise);
  const [wasmModuleLoader, QuickJSAsyncFFI, { QuickJSAsyncWASMModule }] = await Promise.all([
    variant.importModuleLoader().then(smartUnwrap),
    variant.importFFI(),
    import("./module-asyncify-2F223TMF.mjs").then(unwrapTypescript)
  ]);
  const wasmModule = await wasmModuleLoader();
  wasmModule.type = "async";
  const ffi = new QuickJSAsyncFFI(wasmModule);
  return new QuickJSAsyncWASMModule(wasmModule, ffi);
}
function memoizePromiseFactory(fn) {
  let promise;
  return () => {
    return promise ??= fn();
  };
}
function smartUnwrap(val) {
  if (`default` in val) {
    if (`default` in val.default) {
      return val.default.default;
    }
    return val.default;
  }
  return val;
}

// src/vm-interface.ts
function isSuccess(successOrFail) {
  return "error" in successOrFail === false;
}
function isFail(successOrFail) {
  return "error" in successOrFail === true;
}

// src/interrupt-helpers.ts
function shouldInterruptAfterDeadline(deadline) {
  const deadlineAsNumber = typeof deadline === "number" ? deadline : deadline.getTime();
  return function() {
    return Date.now() > deadlineAsNumber;
  };
}

// src/module-test.ts
var TestQuickJSWASMModule = class {
  constructor(parent) {
    this.parent = parent;
    this.contexts = /* @__PURE__ */ new Set();
    this.runtimes = /* @__PURE__ */ new Set();
  }
  newRuntime(options) {
    const runtime = this.parent.newRuntime({
      ...options,
      ownedLifetimes: [
        new Lifetime(void 0, void 0, () => this.runtimes.delete(runtime)),
        ...options?.ownedLifetimes ?? []
      ]
    });
    this.runtimes.add(runtime);
    return runtime;
  }
  newContext(options) {
    const context = this.parent.newContext({
      ...options,
      ownedLifetimes: [
        new Lifetime(void 0, void 0, () => this.contexts.delete(context)),
        ...options?.ownedLifetimes ?? []
      ]
    });
    this.contexts.add(context);
    return context;
  }
  evalCode(code, options) {
    return this.parent.evalCode(code, options);
  }
  disposeAll() {
    const allDisposables = [...this.contexts, ...this.runtimes];
    this.runtimes.clear();
    this.contexts.clear();
    allDisposables.forEach((d) => {
      if (d.alive) {
        d.dispose();
      }
    });
  }
  assertNoMemoryAllocated() {
    const leaksDetected = this.getFFI().QTS_RecoverableLeakCheck();
    if (leaksDetected) {
      throw new QuickJSMemoryLeakDetected("Leak sanitizer detected un-freed memory");
    }
    if (this.contexts.size > 0) {
      throw new QuickJSMemoryLeakDetected(`${this.contexts.size} contexts leaked`);
    }
    if (this.runtimes.size > 0) {
      throw new QuickJSMemoryLeakDetected(`${this.runtimes.size} runtimes leaked`);
    }
  }
  /** @private */
  getFFI() {
    return this.parent.getFFI();
  }
};

export {
  newQuickJSWASMModuleFromVariant,
  newQuickJSAsyncWASMModuleFromVariant,
  memoizePromiseFactory,
  isSuccess,
  isFail,
  shouldInterruptAfterDeadline,
  TestQuickJSWASMModule
};
//# sourceMappingURL=chunk-TUUGOC7J.mjs.map